# -*- coding: utf8 -*-
from __future__ import unicode_literals

import csv

from six import StringIO

from driverng import MessageV2, Worklist
from driverng.splitted import Splitted, String


class CovidResultsBase(Splitted):

    SEPARATOR = ","


class ColibriResult(object):
    FIELDS = [
        # Code barre de la cible
        String("code_barre_cible"),
        # Position du puit
        String("position"),
        # Code barre de l'échantillon
        String("sample_id"),
        # Rang de germe
        String("rang"),
        # Type de puit
        # 1 : pour germe
        # 2 : pour bts
        # 3 : pour sepsityper
        # 4 : pour myco
        String("type_puit"),
    ]

    def parse(self, line):
        fields = self.FIELDS
        for elt, field in enumerate(fields):
            setattr(self, field.name, line[elt])


class ColibriResultsMultiple(list):
    _MESSAGE_TYPE_MAP = {MessageV2.RESULT: ColibriResult}

    def add(self, type_message=None, **kwargs):
        obj = self._MESSAGE_TYPE_MAP[type_message](**kwargs)
        self.append(obj)
        return obj

    def parse(self, data):
        buf = StringIO(data)
        reader = csv.reader(buf, delimiter=str(","))
        for line in reader:
            if not len(line):
                continue
            obj = self.add(type_message=MessageV2.RESULT)
            obj.parse(line)


class Driver(MessageV2):
    SPEC = {"code": "colibri", "label": "Colibri auto création plan de plaque"}

    def parse(self, data):
        obj = ColibriResultsMultiple()
        obj.parse(data)
        return obj

    def decode_message(self, message):
        worklist = Worklist(context="result")
        positions = []
        sample_id = None
        for m in message:
            sample_id = m.code_barre_cible
            sample_position = {
                "code_barre": m.sample_id,
                "position": m.position,
                "type_echantillon": m.type_puit,
            }
            # Pour prévoir son intégration dans MCA le rang est initialisé à 0
            # dans le modèle de donnée de la table BRUKER_POSITION
            if m.rang and int(m.rang) > 0:
                sample_position["rang"] = int(m.rang) - 1
            positions.append(sample_position)
        if sample_id:
            worklist.get_or_create(
                sample_id=sample_id, bruker_positions=positions, type="patient"
            )
        return worklist
